@@ -223,9 +223,13 @@ def info(original_list=[], items=None, list_dependencies=False, list_private=Fal
 
   if not original_list:
     vytools.printer.print_def('  Repositories in this context')
-    repo_versions = vytools.utils.get_all_repo_versions(items)
-    for x in sorted([v for v in repo_versions.values()]):
-      vytools.printer.print_plus('    '+x)
+    for k in items:
+      if k.startswith('repo:'):
+        name = vytools.utils.repo_version_string(items[k])
+        has_stash = vytools.utils.repo_has_stash(items[k]['type'],items[k]['path'])
+        has_unpushed = vytools.utils.repo_has_unpushed(items[k]['type'],items[k]['path'])
+        val = '  |{}|{}|{}| {}'.format('s' if has_stash else ' ','p' if has_unpushed else ' ','+' if name.endswith('+') else ' ',name)
+        vytools.printer.print_plus(val)
   
     if list_images:
       vytools.printer.print_def('  Images on this device')
