@@ -72,12 +72,63 @@ def find_all(items, contextpaths=None):
         vytools.printer.print_fail('Failed to interpret/link compose {c}'.format(c=type_name))
   return success
 
+def _extract_anchors(item, items, object_mods, built, build_level, eppath, propagation):
+  this_anchors = copy.deepcopy(item.get('anchors',{}))
+  for tag in [k for atype in ANCHORTYPES for k,v in this_anchors.items() if v.startswith(atype+':')]: # SORTED TO ENSURE ORDER IS DEFINITION, ARGUMENT, STAGE, ARTIFACT
+    val = this_anchors[tag]
+    if val.startswith('definition:'):
+      obj = propagation['anchors'][tag] if tag in propagation['anchors'] else None
+      obj_mods = object_mods[tag] if object_mods and tag in object_mods else None
+      if obj is not None:
+        propagation['anchors'][tag],deplist_ = vytools.object.expand(obj, val, items, object_mods=obj_mods)
+        propagation['dependencies'] += [d for d in deplist_ if d not in propagation['dependencies']]
+        if propagation['anchors'][tag] is None:
+          vytools.printer.print_fail('Failed to expand anchor "{}" in compose "{}"'.format(tag, item['name']))
+          return False
+      if tag not in propagation['anchors']:
+        vytools.printer.print_warn('Anchor "{}" was not set for use in compose "{}"'.format(tag, item['name']))
+    elif val.startswith('argument:'):
+      if tag not in propagation['anchors']: 
+        propagation['anchors'][tag] = val.replace('argument:','',1)
+    elif val.startswith('stage:'):
+      buildstage = propagation['anchors'][tag] if tag in propagation['anchors'] else val
+      if buildstage.startswith('stage:'):
+        tagged = vytools.stage.build([buildstage], items, propagation['anchors'], built, build_level, jobpath=eppath)
+        if tagged == False: return False
+        for v in tagged.values():
+          propagation['stage_versions'] += [sv for sv in v['stage_versions'] if sv not in propagation['stage_versions']]
+        propagation['anchors'][tag] = tagged[buildstage]['tag']
+      else:
+        propagation['anchors'][tag] = buildstage
+    elif any([val.startswith(v+':') for v in ['directory','file','artifact','vydir','repo']]):
+      if tag not in propagation['anchors']:
+        # propagation['anchors'][tag] = val
+        splitname = val.split(':',1)
+        arttyp = splitname[0]
+        artname = splitname[-1]
+        if len(splitname) == 2 and len(artname) > 0:
+          if arttyp in ['vydir','repo']:
+            artname = vytools.utils.get_thing_path(val, items)
+            if not artname:
+              return False
+          propagation['anchors'][tag] = artname
+          if build_level == -1 and '..' not in artname and eppath and os.path.exists(eppath):
+            artifactpath = os.path.join(eppath, artname)
+            if arttyp in ['artifact','file']:
+              if os.path.isdir(artifactpath):
+                vytools.printer.print_fail('The {} "{}" already exists as a directory. You will need to delete it to continue with this compose file'.format(arttyp,artifactpath))
+                return False
+              with open(artifactpath,'w') as w: w.write('')
+              os.chmod(artifactpath, 0o666)
+            elif arttyp == 'directory':
+              os.makedirs(artifactpath,exist_ok=True)
+  return True
+
 def build(rootcompose, items=None, anchors=None, built=None, build_level=1, object_mods=None, eppath=None, label=None):
   if items is None: items = ITEMS
   if not vytools.utils.ok_dependency_loading('build', rootcompose, items):
     return False
 
-  override_anchors = {} if anchors is None else copy.deepcopy(anchors) # These are the anchor values specified by the episode or otherwise
   if built is None: built = []
   if label is None: label = rootcompose.replace('compose:','') 
   if not utils.exists([rootcompose], items):
@@ -86,73 +137,25 @@ def build(rootcompose, items=None, anchors=None, built=None, build_level=1, obje
     vytools.printer.print_fail('Item {} is not a compose file'.format(rootcompose))
     return False
 
-  stage_versions = []
+  propagation = {
+    'command':[],
+    'stage_versions':[],
+    'dependencies':[rootcompose],
+    'anchors':{} if anchors is None else copy.deepcopy(anchors)
+  }
   item = items[rootcompose]
-  cmd = []
-  dependencies = [rootcompose]
-
-  child_anchors = {} # Populate with the merged/default anchor values from subcomposes [ARGUMENTS]
+  if not _extract_anchors(item, items, object_mods, built, build_level, eppath, propagation):
+    return False
   for sa in item['subcompose']:
     subname = sa['name']
     sublabel = label + '.'+subname.replace('compose:','')
-    subcmds = build(subname, items=items, anchors=anchors, built=built,
+    subcmds = build(subname, items=items, anchors=propagation['anchors'], built=built,
                             build_level=build_level, object_mods=object_mods, eppath=eppath, label=sublabel)
-    if subcmds == False:
-      return False
-    else:
-      child_anchors.update(subcmds['anchors']) # Brutal overwrites not namespacing. I don't think I want namespacing
-      cmd += subcmds['command']
-      dependencies += [d for d in subcmds['dependencies'] if d not in dependencies]
-      stage_versions += [sv for sv in subcmds['stage_versions'] if sv not in stage_versions]
-
-  this_anchors = copy.deepcopy(item.get('anchors',{}))
-  substituted_anchors = {}
-  for tag in [k for atype in ANCHORTYPES for k,v in this_anchors.items() if v.startswith(atype+':')]: # SORTED TO ENSURE ORDER IS DEFINITION, ARGUMENT, STAGE, ARTIFACT
-    val = this_anchors[tag]
-    if val.startswith('definition:'):
-      obj_mods = object_mods[tag] if object_mods and tag in object_mods else None
-      if tag not in override_anchors:
-        vytools.printer.print_warn('Anchor "{}" was not set for use in compose "{}"'.format(tag,rootcompose))
-        substituted_anchors[tag] = {}
-        deplist_ = []
-      else:
-        substituted_anchors[tag],deplist_ = vytools.object.expand(override_anchors[tag],
-            val, items, object_mods=obj_mods)
-      if substituted_anchors[tag] is None:
-        return False
-      dependencies += [d for d in deplist_ if d not in dependencies]
-    elif val.startswith('argument:'):
-      substituted_anchors[tag] = override_anchors[tag] if tag in override_anchors else val.replace('argument:','',1)
-      if tag not in override_anchors: 
-        override_anchors[tag] = substituted_anchors[tag]
-    elif val.startswith('stage:'):
-      buildstage = override_anchors[tag] if tag in override_anchors else val
-      tagged = vytools.stage.build([buildstage], items, override_anchors, built, build_level, jobpath=eppath)
-      if tagged == False: return False
-      for v in tagged.values():
-        stage_versions += [sv for sv in v['stage_versions'] if sv not in stage_versions]
-      substituted_anchors[tag] = tagged[buildstage]['tag']
-    elif any([val.startswith(v+':') for v in ['directory','file','artifact','vydir','repo']]):
-      val_ = override_anchors[tag] if tag in override_anchors else val
-      splitname = val_.split(':',1)
-      arttyp = splitname[0]
-      artname = splitname[-1]
-      if len(splitname) == 2 and len(artname) > 0:
-        if arttyp in ['vydir','repo']:
-          artname = vytools.utils.get_thing_path(val_, items)
-          if not artname:
-            return False
-        substituted_anchors[tag] = artname
-        if build_level == -1 and '..' not in artname and eppath and os.path.exists(eppath):
-          artifactpath = os.path.join(eppath, artname)
-          if arttyp in ['artifact','file']:
-            if os.path.isdir(artifactpath):
-              vytools.printer.print_fail('The {} "{}" already exists as a directory. You will need to delete it to continue with this compose file'.format(arttyp,artifactpath))
-              return False
-            with open(artifactpath,'w') as w: w.write('')
-            os.chmod(artifactpath, 0o666)
-          elif arttyp == 'directory':
-            os.makedirs(artifactpath,exist_ok=True)
+    if subcmds == False: return False
+    propagation['command'] += subcmds['command']
+    for k in ['dependencies','stage_versions']:
+      propagation[k] += [d for d in subcmds[k] if d not in propagation[k]]
+    propagation['anchors'].update(subcmds['anchors'])
 
   if build_level == -1:
     compose_file_name = label + '.yaml'
@@ -161,27 +164,18 @@ def build(rootcompose, items=None, anchors=None, built=None, build_level=1, obje
     with open(compose_pth,'r') as r:
       composition = yaml.safe_load(r.read())
     if 'x-vy' in composition: del composition['x-vy']
-    ok = _recursiv_replace(composition, substituted_anchors, eppath, label)
+    ok = _recursiv_replace(composition, propagation['anchors'], eppath, label)
     if not ok:
       return False
-    child_anchors.update(substituted_anchors)
     
     oktowrite = '..' not in compose_file_name and eppath and os.path.exists(eppath)
     if oktowrite:
-      cfile = os.path.join(eppath, compose_file_name)
-      if not bool(substituted_anchors):
-        shutil.copyfile(compose_pth, cfile)
-      else:
-        with open(cfile,'w') as w:
-          w.write(yaml.safe_dump(composition))
-      cmd = cmd + ['-f',compose_file_name]
+      with open(os.path.join(eppath, compose_file_name),'w') as w:
+        w.write(yaml.safe_dump(composition))
+      propagation['command'] += ['-f',compose_file_name]
   
-  return {
-    'anchors':child_anchors,
-    'command':cmd,
-    'stage_versions':sorted(stage_versions),
-    'dependencies':dependencies
-  }
+  propagation['stage_versions'] = sorted(propagation['stage_versions'])
+  return propagation
 
 def _prefx(key,char,replkeys):
   return [r for r in replkeys if key.startswith(r+char)]
