@@ -85,6 +85,8 @@ def run(epid, compose_name, items=None, anchors=None, clean=False, object_mods=N
   if eppath and os.path.exists(eppath):
     with open(os.path.join(eppath, 'vyanchors.json'),'w') as w:
       json.dump(cbuild['anchors'],w,sort_keys=True,indent=1)
+    with open(os.path.join(eppath, 'vydefinitions.json'),'w') as w:
+      json.dump({x:items[x]['element'] for x in vytools.utils.get_dependencies_of_type(compose_name,['definition'],items,recursive=True)},w,sort_keys=True,indent=1)
 
   cmd = DOCKER_COMPOSE + cbuild['command'] + ['--project-name', epid]
   cmdup = cmd+['up', '--abort-on-container-exit']
