@@ -50,6 +50,34 @@ def find_all_vydirectories(items, contextpaths=None):
   success = search_all(None, None, items, checkvydir=True, contextpaths=contextpaths)
   return success
 
+def get_dependencies_of_type(item, types, items, recursive=True):
+  lst = []
+  if item in items and 'depends_on' in items[item]:
+    for d in items[item]['depends_on']:
+      if d not in lst and any([d.startswith(n+':') for n in types]):
+        lst.append(d)
+      if recursive:
+        lst += [i for i in get_dependencies_of_type(d, types, items, recursive=recursive) if i not in lst]
+  return lst
+
+def repo_has_unpushed(typ, pth):
+  try:
+    if typ == 'hg':
+      return True # TODO Implement for hg
+    elif typ == 'git':
+      return len(subprocess.check_output(['git','log','--branches','--not','--remotes'],cwd=pth).decode('utf8').strip()) > 0
+  except Exception as exc:
+    return True # TODO error message?
+
+def repo_has_stash(typ, pth):
+  try:
+    if typ == 'hg':
+      return True # TODO Implement for hg
+    elif typ == 'git':
+      return len(subprocess.check_output(['git','stash','list'],cwd=pth).decode('utf8').strip()) > 0
+  except Exception as exc:
+    return True # TODO error message?
+
 def repo_version(typ, pth):
   try:
     if typ == 'hg':
